@@ -1,5 +1,5 @@
 package Archive::BagIt;
-our $VERSION = '0.060'; # TRIAL VERSION
+our $VERSION = '0.061'; # TRIAL VERSION
 use strict;
 use warnings;
 use utf8;
@@ -105,7 +105,7 @@ sub _write_baginfo {
     use POSIX;
     my($self, $bagit, %param) = @_;
     open(my $BAGINFO, ">", $bagit."/bag-info.txt") or die("Can't open $bagit/bag-info.txt for writing: $!");
-    $param{'Bagging-Date'} = POSIX::strftime("%F", gmtime(time));
+    $param{'Bagging-Date'} = POSIX::strftime("%Y-%m-%d", gmtime(time));
     $param{'Bag-Software-Agent'} = 'Archive::BagIt <http://search.cpan.org/~rjeschmi/Archive-BagIt>';
     while(my($key, $value) = each(%param)) {
         print($BAGINFO "$key: $value\n");
@@ -386,7 +386,7 @@ Archive::BagIt
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 SYNOPSIS
 
@@ -415,7 +415,7 @@ Archive::BagIt
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 WARNING
 
@@ -570,7 +570,7 @@ Rob Schmidt <rjeschmi@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann.
+This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
@@ -581,7 +581,7 @@ Rob Schmidt <rjeschmi@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann.
+This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
