@@ -15,13 +15,33 @@ use Carp;
 use List::Util qw(uniq);
 use POSIX qw(strftime);
 
-our $VERSION = '0.060'; # TRIAL VERSION
+our $VERSION = '0.061'; # TRIAL VERSION
 
 use Sub::Quote;
 
 my $DEBUG=0;
 
 
+
+around 'BUILDARGS' , sub {
+    my $orig = shift;
+    my $class = shift;
+    if (@_ == 1 && !ref $_[0]) {
+        return $class->$orig(bag_path=>$_[0]);
+    } else {
+        return $class->$orig(@_);
+    }
+};
+
+sub BUILD {
+    my ($self, $args) = @_;
+    return $self->load_plugins(("Archive::BagIt::Plugin::Manifest::MD5", "Archive::BagIt::Plugin::Manifest::SHA512"));
+}
+
+
+###############################################
+
+
 has 'parallel' => ( # used for parallel verify, only usefull if bagits with many files expected!
     is        => 'rw',
     predicate => 'has_parallel',
@@ -29,16 +49,23 @@ has 'parallel' => ( # used for parallel verify, only usefull if bagits with many
 );
 
 
+###############################################
+
 has 'bag_path' => (
     is => 'rw',
 );
 
+
+###############################################
+
 has 'bag_path_arr' => (
     is => 'ro',
     lazy => 1,
     builder => '_build_bag_path_arr',
 );
 
+###############################################
+
 has 'metadata_path' => (
     is=> 'ro',
     lazy => 1,
@@ -51,18 +78,25 @@ sub _build_metadata_path {
 }
 
 
+###############################################
+
 has 'metadata_path_arr' => (
     is =>'ro',
     lazy => 1,
     builder => '_build_metadata_path_arr',
 );
 
+
+###############################################
+
 has 'rel_metadata_path' => (
     is => 'ro',
     lazy => 1,
     builder => '_build_rel_metadata_path',
 );
 
+###############################################
+
 has 'payload_path' => (
     is => 'ro',
     lazy => 1,
@@ -74,36 +108,54 @@ sub _build_payload_path {
     return $self->bag_path."/data";
 }
 
+
+###############################################
+
 has 'payload_path_arr' => (
     is => 'ro',
     lazy => 1,
     builder => '_build_payload_path_arr',
 );
 
+###############################################
+
 has 'rel_payload_path' => (
     is => 'ro',
     lazy => 1,
     builder => '_build_rel_payload_path',
 );
 
+###############################################
+
 has 'checksum_algos' => (
     is => 'ro',
     lazy => 1,
     builder => '_build_checksum_algos',
 );
 
+
+###############################################
+
 has 'bag_version' => (
     is       => 'ro',
     lazy     => 1,
     builder  => '_build_bag_version',
 );
 
+
+###############################################
+
+
 has 'bag_encoding' => (
     is      => 'ro',
     lazy    => 1,
     builder => '_build_bag_encoding',
 );
 
+
+###############################################
+
+
 has 'bag_info' => (
     is        => 'rw',
     lazy      => 1,
@@ -111,12 +163,39 @@ has 'bag_info' => (
     predicate => 1
 );
 
+
+###############################################
+
+
 has 'errors' => (
-    is      => 'rw',
+    is      => 'ro',
     lazy    => 1,
     builder => sub { my $self = shift; return [];},
 );
 
+
+###############################################
+
+has 'digest_callback' => (
+    is      => 'ro',
+    lazy    => 1,
+    builder => sub {
+        my $sub = sub {
+            my ($digestobj, $filename) = @_;
+            open(my $fh, "<:raw", "$filename") or croak ("Cannot open $filename, $!");
+            binmode($fh);
+            my $digest = $digestobj->get_hash_string($fh);
+            close $fh || croak("could not close file '$filename', $!");
+            return $digest;
+        };
+        return $sub;
+    }
+);
+
+
+###############################################
+
+
 # bag_info_by_key()
 sub bag_info_by_key {
     my ($self, $searchkey) = @_;
@@ -132,6 +211,9 @@ sub bag_info_by_key {
 }
 
 
+###############################################
+
+
 sub _replace_bag_info_by_first_match {
     my ($self, $searchkey, $newvalue) = @_;
     my $info = $self->bag_info();
@@ -170,12 +252,6 @@ has 'forced_fixity_algorithm' => (
     builder  => '_build_forced_fixity_algorithm',
 );
 
-has 'bag_checksum' => (
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_bag_checksum',
-);
-
 has 'manifest_files' => (
     is => 'ro',
     lazy => 1,
@@ -188,17 +264,7 @@ has 'tagmanifest_files' => (
     builder => '_build_tagmanifest_files',
 );
 
-has 'manifest_entries' => (
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_manifest_entries',
-);
 
-has 'tagmanifest_entries' => (
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_tagmanifest_entries',
-);
 
 has 'payload_files' => ( # relatively to bagit base
     is => 'ro',
@@ -218,7 +284,9 @@ has 'plugins' => (
 );
 
 has 'manifests' => (
-    is=>'rw',
+    is      => 'rw',
+    lazy    => 1,
+    builder => '_build_manifests'
     #isa=>'HashRef',
 );
 
@@ -228,21 +296,6 @@ has 'algos' => (
 );
 
 
-around 'BUILDARGS' , sub {
-    my $orig = shift;
-    my $class = shift;
-    if (@_ == 1 && !ref $_[0]) {
-        return $class->$orig(bag_path=>$_[0]);
-    } else {
-        return $class->$orig(@_);
-    }
-};
-
-sub BUILD {
-    my ($self, $args) = @_;
-    return $self->load_plugins(("Archive::BagIt::Plugin::Manifest::MD5", "Archive::BagIt::Plugin::Manifest::SHA512"));
-}
-
 sub _build_bag_path_arr {
     my ($self) = @_;
     my @split_path = File::Spec->splitdir($self->bag_path);
@@ -279,20 +332,11 @@ sub _build_checksum_algos {
     return $checksums;
 }
 
-sub _build_bag_checksum {
-  my($self) =@_;
-  my $bagit = $self->{'bag_path'};
-  open(my $SRCFILE, "<:raw",  $bagit."/manifest-md5.txt");
-  my $srchex=Digest::MD5->new->addfile($SRCFILE)->hexdigest;
-  close($SRCFILE);
-  return $srchex;
-}
-
 sub _build_manifest_files {
   my($self) = @_;
   my @manifest_files;
   foreach my $algo (@{$self->checksum_algos}) {
-    my $manifest_file = $self->metadata_path."/manifest-$algo.txt";
+    my $manifest_file = File::Spec->catdir($self->metadata_path, "manifest-$algo.txt");
     if (-f $manifest_file) {
       push @manifest_files, $manifest_file;
     }
@@ -304,7 +348,7 @@ sub _build_tagmanifest_files {
   my ($self) = @_;
   my @tagmanifest_files;
   foreach my $algo (@{$self->checksum_algos}) {
-    my $tagmanifest_file = $self->metadata_path."/tagmanifest-$algo.txt";
+    my $tagmanifest_file = File::Spec->catdir($self->metadata_path,"tagmanifest-$algo.txt");
     if (-f $tagmanifest_file) {
       push @tagmanifest_files, $tagmanifest_file;
     }
@@ -312,52 +356,9 @@ sub _build_tagmanifest_files {
   return \@tagmanifest_files;
 }
 
-sub __build_xxxmanifest_entries {
-  my ($self, $xxmanifestfiles) = @_;
-  my @xxmanifests = @{$xxmanifestfiles};
-  my $xxmanifest_entries = {};
-  my $bag_path=$self->bag_path();
-  foreach my $xxmanifest_file (@xxmanifests) {
-    die("Cannot open $xxmanifest_file: $!") unless (open(my $XXMANIFEST,"<:encoding(utf8)", $xxmanifest_file));
-    my $algo = $xxmanifest_file;
-    $algo =~ s#\\#/#g; # change Backslash to slash to prevent next regex for clashes
-    $algo =~ s#//#/#g; # to fix problems with double path-separators
-    $algo =~ s#^($bag_path/).bagit/#$1#; # FIXME: only for dotbagit-variant, if dotbagit will be outdated, this should be removed
-    $algo =~ s#^$bag_path/##;
-    $algo =~ s#tag(manifest-[a-z0-9]+\.txt)#$1#;
-    $algo =~ s#.*manifest-([a-z0-9]+)\.txt$#$1#;
-    if ($algo =~ m#/#) {
-        die "wrong replacement of path $xxmanifest_file to determine algorithm '$algo' correctly, please contact author";
-    }
-    while (my $line = <$XXMANIFEST>) {
-      chomp($line);
-      my($digest,$file) = split(/\s+/, $line, 2);
-      next unless ((defined $digest) && (defined $file)); # empty lines!
-      $xxmanifest_entries->{$algo}->{$file} = $digest;
-    }
-    close($XXMANIFEST);
-  }
-  return $xxmanifest_entries;
-}
-
-sub _build_tagmanifest_entries {
-  my ($self) = @_;
-  my @tm_files = $self->tagmanifest_files();
-  my $entries = $self->__build_xxxmanifest_entries(@tm_files);
-  return $entries;
-}
-
-sub _build_manifest_entries {
-  my ($self) = @_;
-  my @m_files = $self->manifest_files();
-  my $entries = $self->__build_xxxmanifest_entries(@m_files);
-  return $entries;
-}
-
 sub _build_payload_files{
   my($self) = @_;
   my $payload_dir = $self->payload_path;
-  my $payload_reldir = $self->rel_payload_path;
   my @payload=();
   File::Find::find( sub{
     $File::Find::name = decode ('utf8', $File::Find::name);
@@ -383,8 +384,8 @@ sub _build_payload_files{
 sub __build_read_bagit_txt {
     my($self) = @_;
     my $bagit = $self->metadata_path;
-    my $file = join("/", $bagit, "bagit.txt");
-    open(my $BAGIT, "<", $file) or die("Cannot read '$file': $!");
+    my $file = File::Spec->catdir($bagit, "bagit.txt");
+    open(my $BAGIT, "<", $file) or croak("Cannot read '$file': $!");
     my $version_string = <$BAGIT>;
     my $encoding_string = <$BAGIT>;
     close($BAGIT);
@@ -401,21 +402,21 @@ sub __build_read_bagit_txt {
 sub _build_bag_version {
     my($self) = @_;
     my ($version_string, $encoding_string, $file) = $self->__build_read_bagit_txt();
-    die "Version line missed in '$file" unless defined $version_string;
+    croak "Version line missed in '$file" unless defined $version_string;
     if ($version_string =~ /^BagIt-Version: ([01]\.[0-9]+)$/) {
         return $1
     } else {
         $version_string =~ s/\r/<CR>/;
         $version_string =~ s/^\N{U+FEFF}/<BOM>/;
-        die "Version string '$version_string' of '$file' is incorrect";
+        croak "Version string '$version_string' of '$file' is incorrect";
     };
 }
 
 sub _build_bag_encoding {
     my($self) = @_;
     my ($version_string, $encoding_string, $file) = $self->__build_read_bagit_txt();
-    die "Encoding line missed in '$file" unless defined $encoding_string;
-    die "Encoding '$encoding_string' of '$file' not supported by current Archive::BagIt module!" unless ($encoding_string !~ m/^UTF-8$/);
+    croak "Encoding line missed in '$file" unless defined $encoding_string;
+    croak "Encoding '$encoding_string' of '$file' not supported by current Archive::BagIt module!" unless ($encoding_string !~ m/^UTF-8$/);
     return $encoding_string;
 }
 
@@ -486,9 +487,9 @@ sub _parse_bag_info { # parses a bag-info textblob
 sub _build_bag_info {
     my ($self) = @_;
     my $bagit = $self->metadata_path;
-    my $file = join("/", $bagit, "bag-info.txt");
+    my $file = File::Spec->catdir($bagit, "bag-info.txt");
     if (-e $file) {
-        open(my $BAGINFO, "<", $file) or di("Cannot read $file: $!");
+        open(my $BAGINFO, "<", $file) or croak("Cannot read $file: $!");
         my @lines;
         foreach my $line (<$BAGINFO>) {
             push @lines, $line;
@@ -503,9 +504,7 @@ sub _build_bag_info {
 
 sub _build_non_payload_files {
   my($self) = @_;
-
   my @non_payload = ();
-
   File::Find::find( sub{
     $File::Find::name = decode('utf8', $File::Find::name);
     $_=decode ('utf8', $_);
@@ -548,208 +547,27 @@ sub load_plugins {
 
     return if @plugins == 0;
     foreach my $plugin (@plugins) {
-        load_class ($plugin) or die ("Can't load $plugin");
+        load_class ($plugin) or croak ("Can't load $plugin");
         $plugin->new({bagit => $self});
     }
 
     return 1;
 }
 
-sub __calc_digest {
-    my ($digestobj, $filename) = @_;
-    open(my $fh, '<', $filename) || die("Can't open '$filename', $!");
-    binmode($fh);
-    my $digest = $digestobj->get_hash_string($fh);
-    close $fh || die("could not close file '$filename', $!");
-    return $digest;
-}
-
-
-# calc digest
-# expects digestobj, expected_ref, array_ref of filenames
-# returns arrayref of hashes where each entry has
-# $tmp->{calculated_digest} = $digest;
-# $tmp->{expected_digest} = $expected_digest;
-# $tmp->{filename} = $filename;
-sub calc_digests {
-    my ($self, $bagit, $digestobj, $filenames_ref) = @_;
-    my @digest_hashes;
-    # check if we could use parallel
-    my $is_parallelizeable;
-    if (($self->has_parallel()) && (defined $self->parallel)) {
-        my $err;
-        ($is_parallelizeable, $err) = Class::Load::try_load_class("Parallel::Iterator");
-        if (!$is_parallelizeable) {
-            warn "Class 'Parallel::Iterator' could not be loaded…, $err\n";
-            $self->{parallel} = undef;
-        }
-    }
-    if ($is_parallelizeable) {
-        my $class = Class::Load::load_class("Parallel::Iterator");
-        $class->import( qw(iterate_as_array));
-        @digest_hashes = iterate_as_array(
-            sub {
-                my ($idx, $localname) = @_;
-                my $fullname = $bagit ."/". $localname;
-                my $tmp;
-                $tmp->{calculated_digest} = __calc_digest($digestobj, $fullname);
-                $tmp->{local_name} = $localname;
-                $tmp->{full_name} = $fullname;
-                $tmp;
-            }, $filenames_ref);
-    }
-    else { # fallback to serial processing
-        @digest_hashes = map {
-            my $localname = $_;
-            my $fullname = $bagit ."/". $localname;
-            my $tmp;
-            $tmp->{calculated_digest} = __calc_digest($digestobj, $fullname);
-            $tmp->{local_name} = $localname;
-            $tmp->{full_name} = $fullname;
-            $tmp;
-        } @{$filenames_ref};
-    }
-    return \@digest_hashes;
-}
-
-sub check_if_payload_filepath_violates{
-    my $local_name = shift;
-    # HINT: there is no guarantuee *not* to escape!
-    return
-    ($local_name =~ m/^~/) # Unix Home
-        || ($local_name =~ m#\./#) # Unix, parent dir escape
-        || ($local_name =~ m#^[A-Z]:[\\/]#) # Windows Drive
-        || ($local_name =~ m#^/#) # Unix absolute path
-        || ($local_name =~ m#^$#) # Unix Env
-        || ($local_name =~ m#^\\#) # Windows absolute path
-        || ($local_name =~ m#^%[^%]*%#) # Windows ENV
-        || ($local_name =~ m#^\*#) # Artifact of md5sum-Tool, where ' *' is allowed to separate checksum and file in fixity line
-        || ($local_name =~ m#[<>:"?|]#) # Windows reserved chars
-        || ($local_name =~ m#(CON|PRN|AUX|NUL|COM[1-9]|LPT[1-9])#) # Windows reserved filenames
-    ;
-}
-
-sub normalize_payload_filepath {
-    my $filename = shift;
-    $filename =~ s#[\\](?![/])#/#g; # normalize Windows Backslashes, but only if they are no escape sequences
-    $filename =~ s#%#%25#g; # normalize percent
-    $filename =~ s#\x{0a}#%0A#g; #normalize NEWLINE
-    $filename =~ s#\x{0d}#%0D#g; #normlize CARRIAGE RETURN
-    $filename =~ s# #%20#g; # space
-    $filename =~ s#"##g; # quotes
-    return $filename;
-}
-
-sub _verify_XXX_manifests {
-    my ($self, $xxprefix, $xxmanifest_entries, $files, $return_all_errors) =@_;
-    # Read the manifest file
-    my @payload = @{ $files };
-    my @invalid_messages;
-    my $bagit = $self->bag_path;
-    my $version = $self->bag_version();
-    my $subref_invalid_report_or_die = sub {
-        my $message = shift;
-        if (defined $return_all_errors) {
-            push @invalid_messages, $message;
-        } else {
-            die($message);
-        }
-        return;
-    };
-    foreach my $local_name (@payload) {
-        # local_name is relative to bagit base
-        unless (-r $bagit."/".$local_name) {
-            &$subref_invalid_report_or_die(
-                "cannot read $local_name (bag-path:$bagit)",
-            );
-        }
-    }
-    # Evaluate each file against the manifest
-    foreach my $alg (keys %{$xxmanifest_entries}) {
-        my $manifest_alg = $self->manifests->{$alg};
-        next unless (defined $manifest_alg); # FIXME_ errormessage?
-        my $digestobj = $manifest_alg->algorithm();
-        my $local_xxfilename = "${xxprefix}-${alg}.txt";
-        my $xxfilename = "${bagit}/$local_xxfilename";
-
-        # first check if each file from payload exists in manifest_entries for given alg
-        foreach my $local_name (@payload) {
-            # local_name is relative to bagit base
-            unless (exists $xxmanifest_entries->{$alg}->{normalize_payload_filepath($local_name)}) { # localname as value should exist!
-                &$subref_invalid_report_or_die(
-                    "file '$local_name' found, which is not in '$local_xxfilename' (bag-path:'$bagit')!",
-                );
-            }
-        }
-        # second check if each file from manifest_entries for given alg exists in payload
-        foreach my $local_name (keys %{$xxmanifest_entries->{$alg}}) {
-            if ( # to avoid escapes via manifest-files
-                check_if_payload_filepath_violates( $local_name )
-            ) {
-                &$subref_invalid_report_or_die("file '$local_name' not allowed in '$local_xxfilename' (bag-path:'$bagit'")
-            }
-            else {
-                unless (List::Util::any {normalize_payload_filepath($_) eq $local_name} @payload) {
-                    &$subref_invalid_report_or_die(
-                        "file '$local_name' NOT found, but expected via '$local_xxfilename' (bag-path:'$bagit')!"
-                    );
-                }
-            }
-        }
-        # all preconditions full filled, now calc all digests
-        my $digest_hashes_ref = $self->calc_digests($bagit, $digestobj, \@payload);
-        # compare digests
-        if (defined $digest_hashes_ref && (ref $digest_hashes_ref eq 'ARRAY')) {
-            foreach my $digest_entry (@{$digest_hashes_ref}) {
-                $digest_entry->{expected_digest} = $xxmanifest_entries->{$alg}->{
-                    $digest_entry->{local_name}
-                };
-                if ($digest_entry->{calculated_digest} ne $digest_entry->{expected_digest}) {
-                    &$subref_invalid_report_or_die(
-                        sprintf("file '%s' invalid, digest (%s) calculated=%s, but expected=%s in file '%s'",
-                            $digest_entry->{local_name},
-                            $alg,
-                            $digest_entry->{calculated_digest},
-                            $digest_entry->{expected_digest},
-                            $xxfilename
-                        )
-                    );
-                }
-            }
-        }
-    }
-    if($return_all_errors && (scalar @invalid_messages > 0)) {
-        push @{ $self->{errors} },
-            join("\n\t",
-                "bag verify for bagit version '$version' failed with invalid files:",
-                sort @invalid_messages
-            );
-        return;
-    }
+sub load {
+    my ($self) = @_;
+    # call trigger
+    $self->bag_path;
+    $self->bag_version;
+    $self->bag_encoding;
+    $self->bag_info;
+    $self->payload_path;
+    $self->manifest_files;
+    $self->checksum_algos;
+    $self->tagmanifest_files;
     return 1;
 }
 
-sub _verify_manifests {
-    my ($self, $alg, $return_all_errors) = @_;
-    return $self->_verify_XXX_manifests(
-        "manifest",
-        $self->manifest_entries(),
-        $self->payload_files(),
-        $return_all_errors
-    );
-}
-
-sub _verify_tagmanifests {
-    my ($self, $alg, $return_all_errors) = @_;
-    # filter tagmanifest-files
-    my @non_payload_files = grep { $_ !~ m/tagmanifest-[a-z0-9]+\.txt/} @{ $self->non_payload_files };
-    return $self->_verify_XXX_manifests(
-        "tagmanifest",
-        $self->tagmanifest_entries(),
-        \@non_payload_files,
-        $return_all_errors
-    );
-}
 
 
 
@@ -762,29 +580,37 @@ sub verify_bag {
     my $encoding = $self->bag_encoding(); # to call trigger
     my $baginfo = $self->bag_info(); #to call trigger
     my $forced_fixity_alg = $self->forced_fixity_algorithm()->name();
-    my $fetch_file = $self->metadata_path."/fetch.txt";
-    my $manifest_file = $self->metadata_path."/manifest-$forced_fixity_alg.txt"; # FIXME: use plugin instead
+    my $fetch_file = File::Spec->catdir($self->metadata_path, "fetch.txt");
+    my $manifest_file = File::Spec->catdir($self->metadata_path, "manifest-$forced_fixity_alg.txt");
     my $payload_dir   = $self->payload_path;
     my $return_all_errors = $opts->{return_all_errors};
 
     if (-f $fetch_file) {
-        die("Fetching via file '$fetch_file' is not supported by current Archive::BagIt implementation")
+        croak("Fetching via file '$fetch_file' is not supported by current Archive::BagIt implementation")
     }
-    die("Manifest '$manifest_file' is not a regular file or does not exist for given bagit version '$version'") unless -f ($manifest_file);
-    die("Payload-directory '$payload_dir' is not a directory or does not exist") unless -d ($payload_dir);
+    croak("Manifest '$manifest_file' is not a regular file or does not exist for given bagit version '$version'") unless -f ($manifest_file);
+    croak("Payload-directory '$payload_dir' is not a directory or does not exist") unless -d ($payload_dir);
 
     unless ($version > .95) {
-        die ("Bag Version $version is unsupported");
+        croak ("Bag Version $version is unsupported");
     }
 
     # check forced fixity
 
-    my $ret_manifests = $self->_verify_manifests($forced_fixity_alg, $return_all_errors);
-    my $ret_tagmanifests = $self->_verify_tagmanifests($forced_fixity_alg, $return_all_errors);
-    if ($ret_manifests && $ret_tagmanifests) {
-        return 1;
-    } else {
-        die join("\n", @{ $self->errors });
+    my @errors;
+    foreach my $algorithm ( keys %{ $self->manifests }) {
+        my $res = $self->manifests->{$algorithm}->verify_manifest($self->payload_files, $return_all_errors);
+        if ((defined $res) && ($res ne "1")) { push @errors, $res; }
+    }
+    foreach my $algorithm ( keys %{ $self->manifests }) {
+        my $res = $self->manifests->{$algorithm}->verify_tagmanifest($self->non_payload_files, $return_all_errors);
+        if ((defined $res) && ($res ne "1")) { push @errors, $res; }
+    }
+    push @{$self->{errors}}, @errors;
+    my $err = $self->errors();
+    my @err =  @{ $err };
+    if (scalar( @err ) > 0) {
+        croak join("\n","bag verify for bagit version '$version' failed with invalid files.", @err);
     }
     return 1;
 }
@@ -796,7 +622,7 @@ sub calc_payload_oxum {
     my $octets=0;
     my $streamcount = scalar @payload;
     foreach my $local_name (@payload) {# local_name is relative to bagit base
-        my $file = $self->bag_path()."/$local_name";
+        my $file = File::Spec->catdir($self->bag_path(), $local_name);
         my $sb = stat($file);
         $octets += $sb->size;
     }
@@ -818,7 +644,8 @@ sub calc_bagsize {
 sub create_bagit {
     my($self) = @_;
     my $metadata_path = $self->metadata_path();
-    open(my $BAGIT, ">", "$metadata_path/bagit.txt") or die("Can't open $metadata_path/bagit.txt for writing: $!");
+    my $bagit_path = File::Spec->catdir( $metadata_path, "bagit.txt");
+    open(my $BAGIT, ">", $bagit_path) or croak("Can't open $bagit_path for writing: $!");
     print($BAGIT "BagIt-Version: 1.0\nTag-File-Character-Encoding: UTF-8");
     close($BAGIT);
     return 1;
@@ -834,7 +661,8 @@ sub create_baginfo {
     $self->_add_or_replace_bag_info('Bag-Size', $self->calc_bagsize());
     # The RFC does not allow reordering:
     my $metadata_path = $self->metadata_path();
-    open(my $BAGINFO, ">", "$metadata_path/bag-info.txt") or die("Can't open $metadata_path/bag-info.txt for writing: $!");
+    my $bag_info_path = File::Spec->catdir( $metadata_path, "bag-info.txt");
+    open(my $BAGINFO, ">", $bag_info_path) or croak("Can't open $bag_info_path for writing: $!");
     foreach my $entry (@{ $self->bag_info() }) {
         my %tmp = %{ $entry };
         my ($key, $value) = each %tmp;
@@ -868,9 +696,9 @@ sub store {
 sub init_metadata {
     my ($class, $bag_path) = @_;
     $bag_path =~ s#/$##; # replace trailing slash
-    unless ( -d $bag_path) { die ( "source bag directory '$bag_path' doesn't exist"); }
+    unless ( -d $bag_path) { croak ( "source bag directory '$bag_path' doesn't exist"); }
     my $self = $class->new(bag_path=>$bag_path);
-    warn "no payload path" if ! -d $self->payload_path;
+    carp "no payload path" if ! -d $self->payload_path;
     unless ( -d $self->payload_path) {
         rename ($bag_path, $bag_path.".tmp");
         mkdir  ($bag_path);
@@ -898,7 +726,7 @@ sub make_bag {
   my ($class, $bag_path) = @_;
   my $isa = ref $class;
   if ($isa eq "Archive::BagIt::Base") { # not a class, but an object!
-    die "make_bag() only a class subroutine, not useable with objects. Try store() instead!\n";
+    croak "make_bag() only a class subroutine, not useable with objects. Try store() instead!\n";
   }
   my $self = $class->init_metadata($bag_path);
   return $self;
@@ -921,7 +749,7 @@ Archive::BagIt::Base
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 SYNOPSIS
 
@@ -955,13 +783,23 @@ You only need to know the following methods first:
     my $bag4 = Archive::BagIt::Base->new($bag_dir);
     my $is_valid2 = $bag4->verify_bag( {report_all_errors => 1} );
 
+=head2 read a BagIt-dir, change something, store
+
+Because all methods operate lazy, you should ensure to parse parts of the bag *BEFORE* you modify it.
+Otherwise it will be overwritten!
+
+    use Archive::BagIt::Base;
+    my $bag5 = Archive::BagIt::Base->new($bag_dir); # lazy, nothing happened
+    $bag5->load(); # this updates the object representation by parsing the given $bag_dir
+    $bag5->store(); # this writes the bag new
+
 =head1 NAME
 
 Archive::BagIt::Base
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 NAME
 
@@ -1028,22 +866,59 @@ See L<https://datatracker.ietf.org/doc/rfc8493/?include_text=1> for details.
 
 =over
 
-=item Add support for non-Unix based filesystems
-
 =item enhanced testsuite
 
-=item improved plugin mechanism
-
 =item reduce complexity
 
 =item use modern perl code
 
 =item add code to easily update outdated Bags to v1.0
 
-=item add more ecamples in documentation
-
 =back
 
+=head1 FAQ
+
+=head2 How to access the manifest-entries directly?
+
+Try this:
+
+   foreach my $algorithm ( keys %{ $self->manifests }) {
+       my $entries_ref = $self->manifests->{$algorithm}->manifest_entries();
+       # $entries_ref returns a hashref of form:
+       # $entries_ref->{$algorithm}->{$file} = $digest;
+   }
+
+Similar for tagmanifests
+
+=head2 How fast is C<Archive::BagIt::Fast>?
+
+It depends. On my system with SSD and a 38MB bag with 48 payload files the results for C<verify_bag()> are:
+
+                  Rate BaseParallel FastParallel         Base         Fast
+   BaseParallel 2.46/s           --          -2%         -52%         -57%
+   FastParallel 2.52/s           2%           --         -51%         -56%
+   Base         5.10/s         107%         102%           --         -10%
+   Fast         5.69/s         131%         125%          11%           --
+
+On network filesystem (CIFS, 1GB) with same Bag:
+
+                  Rate FastParallel         Fast BaseParallel         Base
+   FastParallel 1.97/s           --         -10%         -15%         -20%
+   Fast         2.20/s          12%           --          -6%         -11%
+   BaseParallel 2.33/s          18%           6%           --          -6%
+   Base         2.48/s          26%          13%           6%           --
+
+But you should measure which variant is best for you. In general the default C<Archive::BagIt::Base> is fast enough.
+
+=head2 How to update an old bag of version v0.97 to v1.0?
+
+You could try this:
+
+   use Archive::BagIt::Base;
+   my $bag=Archive::BagIt::Base->new( $my_old_bag_filepath );
+   $bag->load();
+   $bag->store();
+
 =head1 METHODS
 
 =head2 Constructor
@@ -1078,22 +953,81 @@ The arguments are:
 
 =back
 
+The bag object will use $bag_dir, BUT an existing $bag_dir is not read. If you use C<store()> an existing bag will be overwritten!
+
+See C<load()> if you want to parse/modify an existing bag.
+
+=head2 has_parallel()
+
+to check if parallelization is possible.
+
+=head2 bag_path([$new_value])
+
+Getter/setter for bag path
+
+=head2 metadata_path()
+
+Getter for metadata path
+
+=head2 payload_path()
+
+Getter for payload path
+
+=head2 checksum_algos()
+
+Getter for registered Checksums
+
+=head2 bag_version()
+
+Getter for bag version
+
+=head2 bag_encoding()
+
+Getter for bag encoding.
+
+HINT: the current version of Archive::BagIt::Base only supports UTF-8, but the method could return other values depending on given Bags.
+
+=head2 bag_info([$new_value])
+
+Getter/Setter for bag info. Expects/returns an array of HashRefs implementing simple key-value pairs.
+
+HINT: RFC8493 does not allow *reordering* of entries!
+
+=head2 errors()
+
+Getter to return collected errors after a C<verify_bag()> call with Option C<report_all_errors>
+
+=head2 digest_callback()
+
+This method could be reimplemented by derived classes to handle fixity checks in own way. The
+getter returns an anonymous function with following interface:
+
+   my $digest = $self->digest_callback;
+   &$digest( $digestobject, $filename);
+
+This anonymous function MUST use the C<get_hash_string()> function of the C<Archive::BagIt::Role::Algorithm> role,
+which is implemented by each C<Archive::BagIt::Plugin::Algorithm::XXXX> module.
+
+See C<Archive::BagIt::Fast> for details.
+
+=head2 bag_info_by_key($searchkey)
+
+TODO: Returns all values which match $searchkeyx
+
 =head2 load_plugins
 
 As default SHA512 and MD5 will be loaded and therefore used. If you want to create a bag only with one or a specific
 checksum-algorithm, you could use this method to (re-)register it. It expects list of strings with namespace of type:
 Archive::BagIt::Plugin::Algorithm::XXX where XXX is your chosen fixity algorithm.
 
-=head2 calc_digests($bagit, $digestobj, $filenames_ref, $opts)
+=head2 load()
 
-Method to calculate and return all digests for a a list of files using a Digest-object. This method will be overwritten by C<Archive::BagIt::Fast>.
-
-If object ist build with option C<parallel>, the digests will be build in parallel.
+Triggers loading of an existing bag
 
 =head2 verify_bag($opts)
 
 A method to verify a bag deeply. If C<$opts> is set with C<{return_all_errors}> all fixity errors are reported.
-The default ist to die with error message if any error is detected.
+The default ist to croak with error message if any error is detected.
 
 HINT: You might also want to check Archive::BagIt::Fast to see a more direct way of accessing files (and thus faster).
 
@@ -1111,7 +1045,7 @@ creates a bagit.txt file
 
 =head2 create_baginfo()
 
-creates a bag-info.tx
+creates a bag-info.txt file
 
 =head2 store()
 
@@ -1152,7 +1086,7 @@ Rob Schmidt <rjeschmi@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann.
+This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
@@ -1163,7 +1097,7 @@ Rob Schmidt <rjeschmi@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann.
+This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
