@@ -1,18 +1,17 @@
-use strict;
-use warnings;
-
 package Archive::BagIt::DotBagIt;
 
-our $VERSION = '0.060'; # TRIAL VERSION
-
+our $VERSION = '0.061'; # TRIAL VERSION
+use strict;
+use warnings;
 use Sub::Quote;
+use Carp;
+use File::Spec;
 use Moo;
-
 extends "Archive::BagIt::Base";
 
 
 before BUILDARGS => sub {
-    warn "The module Archive::BagIt::DotBagIt is marked as deprecated and will be erased in next releases!";
+    carp "The module Archive::BagIt::DotBagIt is marked as deprecated and will be erased in next releases!";
 };
 
 has 'metadata_path' => (
@@ -24,7 +23,7 @@ has 'metadata_path' => (
 sub _build_metadata_path {
     my ($self) = @_;
     my $bag_path = $self->bag_path();
-    return "$bag_path/.bagit";
+    return File::Spec->catdir($bag_path, ".bagit");
 }
 
 has 'payload_path' => (
@@ -52,7 +51,7 @@ Archive::BagIt::DotBagIt
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 NAME
 
@@ -60,7 +59,7 @@ Archive::BagIt::DotBagIt
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 NAME
 
@@ -88,7 +87,7 @@ Rob Schmidt <rjeschmi@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann.
+This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
@@ -99,7 +98,7 @@ Rob Schmidt <rjeschmi@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann.
+This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
