@@ -1,11 +1,9 @@
+#ABSTRACT: The default MD5 algorithm plugin
+package Archive::BagIt::Plugin::Algorithm::MD5;
 use strict;
 use warnings;
-
-#ABSTRACT: The default MD5 algorithm plugin
-
-package Archive::BagIt::Plugin::Algorithm::MD5;
-
 use Moo;
+use Carp;
 use namespace::autoclean;
 
 with 'Archive::BagIt::Role::Algorithm';
@@ -30,7 +28,7 @@ has '_digest' => (
 
 sub _build_digest_md5 {
     my ($self) = @_;
-    my $digest_md5 = new Digest::MD5;
+    my $digest_md5 = Digest::MD5->new();
     return $digest_md5;
 }
 
@@ -39,6 +37,9 @@ sub get_hash_string {
     my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
         $atime,$mtime,$ctime,$blksize,$blocks)
         = stat $fh;
+    if ((!defined $blksize) || (length($blksize) < 1) || ($blksize <1)) {
+        $blksize = 8192;
+    }
     my $buffer;
     while (read($fh, $buffer, $blksize)) {
         $self->_digest->add($buffer);
@@ -49,10 +50,10 @@ sub get_hash_string {
 
 sub verify_file {
     my ($self, $filename) = @_;
-    open(my $fh, '<', $filename) || die ("Can't open '$filename', $!");
+    open(my $fh, '<', $filename) || croak ("Can't open '$filename', $!");
     binmode($fh);
     my $digest = $self->get_hash_string($fh);
-    close $fh || die("could not close file '$filename', $!");
+    close $fh || croak("could not close file '$filename', $!");
     return $digest;
 }
 __PACKAGE__->meta->make_immutable;
@@ -70,7 +71,7 @@ Archive::BagIt::Plugin::Algorithm::MD5 - The default MD5 algorithm plugin
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 NAME
 
@@ -78,7 +79,7 @@ Archive::BagIt::Plugin::Algorithm::MD5 - The default MD5 algorithm plugin
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 AVAILABILITY
 
@@ -102,7 +103,7 @@ Rob Schmidt <rjeschmi@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann.
+This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
@@ -113,7 +114,7 @@ Rob Schmidt <rjeschmi@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann.
+This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
