@@ -1,13 +1,10 @@
+#ABSTRACT: The default SHA algorithms plugin
+package Archive::BagIt::Plugin::Algorithm::SHA512;
 use strict;
 use warnings;
-
-#ABSTRACT: The default SHA algorithms plugin
-
-package Archive::BagIt::Plugin::Algorithm::SHA512;
-
+use Carp;
 use Moo;
 use namespace::autoclean;
-
 with 'Archive::BagIt::Role::Algorithm';
 
 has '+plugin_name' => (
@@ -39,6 +36,9 @@ sub get_hash_string {
     my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
         $atime,$mtime,$ctime,$blksize,$blocks)
         = stat $fh;
+    if ((!defined $blksize) || (length($blksize) < 1) || ($blksize <1)) {
+        $blksize = 8192;
+    }
     my $buffer;
     while (read($fh, $buffer, $blksize)) {
         $self->_digest->add($buffer);
@@ -48,10 +48,10 @@ sub get_hash_string {
 
 sub verify_file {
     my ($self, $filename) = @_;
-    open(my $fh, '<', $filename) || die ("Can't open '$filename', $!");
+    open(my $fh, '<', $filename) || croak ("Can't open '$filename', $!");
     binmode($fh);
     my $digest = $self->get_hash_string($fh);
-    close $fh || die("could not close file '$filename', $!");
+    close $fh || croak("could not close file '$filename', $!");
     return $digest;
 }
 __PACKAGE__->meta->make_immutable;
@@ -69,7 +69,7 @@ Archive::BagIt::Plugin::Algorithm::SHA512 - The default SHA algorithms plugin
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 NAME
 
@@ -77,7 +77,7 @@ Archive::BagIt::Plugin::Algorithm::SHA512 - The default SHA algorithms plugin
 
 =head1 VERSION
 
-version 0.060
+version 0.061
 
 =head1 AVAILABILITY
 
@@ -101,7 +101,7 @@ Rob Schmidt <rjeschmi@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann.
+This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
@@ -112,7 +112,7 @@ Rob Schmidt <rjeschmi@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann.
+This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
